/**
 * @file MbusDataTableInterface.hpp
 *
 * @if NOTICE
 *
 * THIS FILE IS PART OF PROPRIETARY SOFTWARE COPYRIGHTED BY PROCONX PTY LTD
 * AND YOU NEED A LICENSE OR WRITTEN PERMISSION TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSDATATABLEINTERFACE_H_INCLUDED
#define _MBUSDATATABLEINTERFACE_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif



/*****************************************************************************
 * MasterInfo declaration
 *****************************************************************************/

struct MasterInfo
{
   enum
   {
      RTU, ASCII, TCP
   };
   int protocol;
   int slaveId;
   char *masterIpAddr;
   unsigned short transactionId;
};


/*****************************************************************************
 * MbusDataTableInterface class declaration
 *****************************************************************************/

/**
 * @brief This class defines the interface between a Modbus slave Server
 * Engine and your application. Descendants of this class are referred to as
 * Data Providers.
 *
 * To create an application specific Data Provider derive a new class from
 * MbusDataTableInterface and override the required data access methods.
 *
 * @ingroup dataprovider
 * @see mbusslave
 * @see MbusRtuSlaveProtocol, MbusAsciiSlaveProtocol, MbusTcpSlaveProtocol
 */
class MbusDataTableInterface
{

  protected:

   MbusDataTableInterface()
   {
   }


   MasterInfo *masterInfoPtr;


  public:

   virtual ~MbusDataTableInterface()
   {
   }


   /**
    * @name Data Access Methods for Table 4:00000 (Holding Registers)
    *
    * Data Access Methods to support read and write of output registers
    * (holding registers) in table 4:00000.
    *
    * This table is accessed by the following Modbus functions:
    *
    * - Modbus function 16 (10 hex), Preset Multiple Registers/Write
    * Multiple Registers
    * - Modbus function 3 (03 hex), Read Holding Registers/Read Multiple
    * Registers
    * - Modbus function 6 (06 hex), Preset Single Register/Write Single
    * Register.
    * - Modbus function 22 (16 hex), Mask Write Register.
    * - Modbus function 23 (17 hex), Read/Write Registers.
    *
    * @ingroup dataprovider
    */
   //@{

   /**
    * Override this method to implement a Data Provider function to read
    * Holding Registers.
    *
    * When a slave receives a poll request for the 4:00000 data table it
    * calls this method to retrieve the data.
    *
    * A simple implementation which holds the application data in an array
    * of shorts (<code>short regData[0x10000]</code>) could be:
    * @code
   int readHoldingRegistersTable(int startRef, short regArr[], int refCnt)
   {
      startRef--; // Adjust Modbus reference counting

      if (startRef + refCnt > (int) sizeof(regData) / sizeof(short))
         return 0;

      memcpy(regArr, &regData[startRef], refCnt * sizeof(short));
      return 1;
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param regArr Buffer which has to be filled with the reply data
    * @param refCnt Number of registers to be retrieved (Range: 0 - 125)
    *
    * @retval 1 Indicate a successful access and that valid reply data is
    * contained in regArr. The Server Engine will reply the data passed in
    * regArr to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * Yes
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int readHoldingRegistersTable(int startRef,
                                         short regArr[],
                                         int refCnt)
   {
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }


   /**
    * Override this method to implement a Data Provider function to write
    * Holding Registers.
    *
    * When a slave receives a write request for the 4:00000 data table it
    * calls this method to pass the data to the application.
    *
    * A simple implementation which holds the application data in an array
    * of shorts (<code>short regData[0x10000]</code>) could be:
    * @code
   int writeHoldingRegistersTable(int startRef, const short regArr[], int refCnt)
   {
      startRef--; // Adjust Modbus reference counting

      if (startRef + refCnt > (int) sizeof(regData) / sizeof(short))
         return 0;

      memcpy(&regData[startRef], regArr, refCnt * sizeof(short));
      return 1;
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param regArr Buffer which contains the received data
    * @param refCnt Number of registers received (Range: 0 - 125)
    *
    * @retval 1 Indicate a successful access. The Server Engine will send a
    * positive reply to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * Yes
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int writeHoldingRegistersTable(int startRef,
                                          const short regArr[],
                                          int refCnt)
   {
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }

   //@}


   /**
    * @name Data Access Methods for Table 3:00000 (Input Registers)
    *
    * Data Access Methods to support read of input registers in table
    * 3:00000.
    *
    * This table is accessed by the following Modbus functions:
    *
    * - Modbus function 4 (04 hex), Read Input Registers.
    *
    * @note Input registers cannot be written
    *
    * @ingroup dataprovider
    */
   //@{

   /**
    * Override this method to implement a Data Provider function to read
    * Input Registers.
    *
    * When a slave receives a poll request for the 3:00000 data table it
    * calls this method to retrieve the data.
    *
    * A simple and very common implementation is to map the Input Registers
    * to the same address space than the Holding Registers table:
    * @code
   int readInputRegistersTable(int startRef, short regArr[], int refCnt)
   {
      return readHoldingRegistersTable(startRef, regArr, refCnt);
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param regArr Buffer which has to be filled with the reply data
    * @param refCnt Number of registers to be retrieved (Range: 0 - 125)
    *
    * @retval 1 Indicate a successful access and that valid reply data is
    * contained in regArr. The Server Engine will reply the data passed in
    * regArr to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int readInputRegistersTable(int startRef,
                                       short regArr[],
                                       int refCnt)
   {
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }

   //@}


   /**
    * @name Data Access Methods for Table 0:00000 (Coils)
    *
    * Data Access Methods to support read and write of discrete outputs
    * (coils) in table 0:00000.
    *
    * This table is accessed by the following Modbus functions:
    *
    * - Modbus function 1 (01 hex), Read Coil Status/Read Coils.
    * - Modbus function 5 (05 hex), Force Single Coil/Write Coil.
    * - Modbus function 15 (0F hex), Force Multiple Coils.
    *
    * @ingroup dataprovider
    */
   //@{

   /**
    * Override this method to implement a Data Provider function to read
    * Coils.
    *
    * When a slave receives a poll request for the 0:00000 data table it
    * calls this method to retrieve the data.
    *
    * A simple implementation which holds the boolean application data in
    * an array of chars (<code>char bitData[2000]</code>) could be:
    * @code
   int readCoilsTable(int startRef, char bitArr[], int refCnt)
   {
      startRef--; // Adjust Modbus reference counting

      if (startRef + refCnt > (int) sizeof(bitData) / sizeof(char))
         return 0;

      memcpy(bitArr, &bitData[startRef], refCnt * sizeof(char));
      return 1;
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param bitArr Buffer which has to be filled with the reply data. Each
    * char represents one coil!
    * @param refCnt Number of coils to be retrieved (Range: 0 - 2000)
    *
    * @retval 1 Indicate a successful access and that valid reply data is
    * contained in regArr. The Server Engine will reply the data passed in
    * regArr to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int readCoilsTable(int startRef,
                              char bitArr[],
                              int refCnt)
   {
      (void) startRef;
      (void) bitArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }


   /**
    * Override this method to implement a Data Provider function to write
    * Coils.
    *
    * When a slave receives a write request for the 0:00000 data table it
    * calls this method to pass the data to the application.
    *
    * A simple implementation which holds the boolean application data in
    * an array of chars (<code>char bitData[2000]</code>) could be:
    * @code
   int writeCoilsTable(int startRef, const char bitArr[], int refCnt)
   {
      startRef--; // Adjust Modbus reference counting

      if (startRef + refCnt > (int) sizeof(bitData) / sizeof(char))
         return 0;

      memcpy(&bitData[startRef], bitArr, refCnt * sizeof(char));
      return 1;
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param bitArr Buffer which contains the received data. Each
    * char repesents one coil!
    * @param refCnt Number of coils received (Range: 0 - 2000)
    *
    * @retval 1 Indicate a successful access. The Server Engine will send a
    * positive reply to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int writeCoilsTable(int startRef,
                               const char bitArr[],
                               int refCnt)
   {
      (void) startRef;
      (void) bitArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }

   //@}


   /**
    * @name Data Access Methods for Table 1:00000 (Input Discretes)
    *
    * Data Access Methods to support read discrete inputs
    * (input status) in table 1:00000.
    *
    * This table is accessed by the following Modbus functions:
    *
    * - Modbus function 2 (02 hex), Read Inputs Status/Read Input Discretes.
    *
    * @note Input Discretes cannot be written
    *
    * @ingroup dataprovider
    */
   //@{

   /**
    * Override this method to implement a Data Provider function to read
    * Coils.
    *
    * When a slave receives a poll request for the 1:00000 data table it
    * calls this method to retrieve the data.
    *
    * A simple and very common implementation is to map the Input Discretes
    * to the same address space than the Coils table:
    * @code
   int readInputDiscretesTable(int startRef, char bitArr[], int refCnt)
   {
      return readCoilsTable(startRef, bitArr, refCnt);
   }
    *
    * @endcode
    *
    * @param startRef Start register (Range: 1 - 65536)
    * @param bitArr Buffer which has to be filled with the reply data. Each
    * char repesents one discrete!
    * @param refCnt Number of discretes to be retrieved (Range: 0 - 2000)
    *
    * @retval 1 Indicate a successful access and that valid reply data is
    * contained in regArr. The Server Engine will reply the data passed in
    * regArr to the master.
    * @retval 0 Indicate that access has been denied or is out of
    * range. The Server Engine will reply to the master with an exception
    * reply message
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Returns 0 which indicates to Server Engine that this address range is
    * unsupported.
    */
   virtual int readInputDiscretesTable(int startRef,
                                       char bitArr[],
                                       int refCnt)
   {
      (void) startRef;
      (void) bitArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }

   //@}


   /**
    * @name Data Access Synchronisation Functions
    *
    * Implementation of these functions may only be required in
    * multithreaded applications, if you are running the server loop in a
    * separate thread and in addition require data consistency over a block
    * of Modbus registers. Data consistency within a single register is
    * always maintained if the code executes on a 16-bit or 32-bit machine,
    * because the CPU is accessing these data types atomically.
    *
    * @ingroup dataprovider
    */
   //@{

   /**
    * You can override this method to implement a semaphore locking
    * mechanism to synchronise data access. This is not needed in single
    * threaded applications but may be necessary in multithreaded
    * applications if you are running the server loop in a separate thread
    * and require data consistency over a block of Modbus registers. Data
    * consistency within a single register is always maintained if the code
    * executes on a 16-bit or 32-bit machine, because the CPU is accessing
    * these data types atomically.
    *
    * This function is called by the server before calling any data read or
    * write functions.
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Empty
    */
   virtual void lock()
   {
   }


   /**
    * You can override this method to implement a semaphore un-locking
    * mechanism to synchronise data access. This is not needed in single
    * threaded applications but may be necessary in multithreaded
    * applications if you are running the server loop in a separate thread
    * and require data consistency over a block of Modbus registers. Data
    * consistency within a single register is always maintained if the code
    * executes on a 16-bit or 32-bit machine, because the CPU is accessing
    * these data types atomically.
    *
    * This function is called by the server after calling any data read or
    * write functions.
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Empty
    */
   virtual void unlock()
   {
   }

   //@}


   /**
    * @name Auxiliary Functions
    * @ingroup dataprovider
    */
   //@{

   /**
    * Override this method to implement a function to handle master poll
    * time-outs.
    *
    * A master should poll a slave cyclically. If no master is polling
    * within the time-out period this method is called. A slave can take
    * certain actions if the master has lost connection, e.g. go into a
    * fail-safe state.
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Empty
    */
   virtual void timeOutHandler()
   {
   }


   /**
    * Override this method to implement a function with reports the eight
    * exception status coils (bits) within the slave device.
    *
    * The exception status coils are device specific and usually used to
    * report a device' principal status or a device' major failure codes as a
    * 8-bit word.
    *
    * @return Exception status byte
    *
    * @par Required:
    * No
    * @par Default Implementation:
    * Returns 0 as exception status byte.
    */
   virtual char readExceptionStatus()
   {
      return 0;
   }

   //@}


   virtual int readEnronRegistersTable(int startRef,
                                       long regArr[],
                                       int refCnt)
   {
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // 0 indicates we do not support ENRON 32-bit values
   }


   virtual int writeEnronRegistersTable(int startRef,
                                        const long regArr[],
                                        int refCnt)
   {
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // 0 indicates we do not support ENRON 32-bit values
   }


   virtual int readFileRecord(int refType, int fileNo, int startRef,
                              short regArr[], int refCnt)
   {
      (void) refType;
      (void) fileNo;
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }


   virtual int writeFileRecord(int refType, int fileNo, int startRef,
                               short regArr[], int refCnt)
   {
      (void) refType;
      (void) fileNo;
      (void) startRef;
      (void) regArr;
      (void) refCnt;
      return 0; // Slave will send an exception for unsupported range
   }


   virtual int getSlaveId(char bufferArr[], int maxBufSize)
   {
      (void) maxBufSize;
      bufferArr[0] = '?';
      return 1;
   }


   virtual int getRunIndicatorStatus()
   {
      return 0; // 0 is off
   }


   virtual int getDeviceIdObject(int objId, char bufferArr[], int maxBufSize)
   {
      (void) objId;
      (void) bufferArr;
      (void) maxBufSize;
      return 0; // Slave will send an exception for unsupported ID
   }


   void setMasterInfo(MasterInfo *masterInfoPtr)
   {
      this->masterInfoPtr = masterInfoPtr;
   }


   MasterInfo *getMasterInfo()
   {
      return this->masterInfoPtr;
   }


};


#endif // ifdef ..._H_INCLUDED

