/**
 * @file MbusRtuSlaveProtocol.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSRTUSLAVEPROTOCOL_H_INCLUDED
#define _MBUSRTUSLAVEPROTOCOL_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif

// Package header
#include "MbusSerialSlaveProtocol.hpp"


/*****************************************************************************
 * MbusRtuSlaveProtocol class declaration
 *****************************************************************************/

/**
 * This class realises the server-side of the Modbus RTU slave protocol.
 *
 * It provides functions to start-up and to execute the server engine which
 * includes opening and closing of the serial port. Upon receipt of a valid
 * master query the server engine calls Data Provider methods to exchange
 * data with the user application.
 * For a more detailed description which Modbus data and
 * control functions have been implemented in the server engine see section
 * @ref mbusslave.
 *
 * It is possible to instantiate multiple instances for establishing
 * multiple connections on different serial ports, however they
 * must be executed in separate threads.
 *
 * @ingroup mbusslaveserial
 * @see mbusslave, mbusslaveserial
 * @see MbusDataTableInterface, MbusAsciiSlaveProtocol, MbusTcpSlaveProtocol
 */
class
#if defined (_WINDLL) || defined(__DLL__)
   __declspec(dllexport)
#endif
MbusRtuSlaveProtocol: public MbusSerialSlaveProtocol
{

  private:

   enum
   {
      SER_DATABITS_7 = 7     // Hide this
   };

   enum
   {
      // RTU header length is 2 bytes address/function
      HDR_LEN = 2 ,
      // RTU frame length is header + 2 bytes crc
      FRAME_LEN = HDR_LEN + 2,
      // RTU exception message length is frame + 1 byte exception code
      EXC_MSG_LEN = FRAME_LEN + 1,
      // RTU max. message size is data size + frame length
      MAX_MSG_SIZE = MAX_DATA_SIZE + FRAME_LEN
   };

   int frameSilence;
   char bufferArr[MAX_MSG_SIZE];


  public:

   MbusRtuSlaveProtocol();

   int startupServer(const TCHAR * const portName,
                     long baudRate, int dataBits,
                     int stopBits, int parity);

   int serverLoop();


  private:

   int replyMessage(int replyLen);


  private:

   // Disable default operator and copy constructor
   MbusRtuSlaveProtocol &operator= (MbusRtuSlaveProtocol &);
   MbusRtuSlaveProtocol (const MbusRtuSlaveProtocol &);

};


#endif // ifdef ..._H_INCLUDED
