/**
 * @file MbusSlaveServer.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSSLAVESERVER_H_INCLUDED
#define _MBUSSLAVESERVER_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif

// Platform header
#include <stdlib.h>

// Package header
#include "BusProtocolErrors.h"
#include "MbusDataTableInterface.hpp"


/*****************************************************************************
 * MbusSlaveServer class declaration
 *****************************************************************************/

/**
 * @brief Base class which implements the Modbus server engine
 *
 * This class realises the server engine. The server engines processes
 * Modbus messages, parses the function codes and upon receipt of a valid
 * master query it calls Data Provider methods to exchange data with the
 * user application.  For a more detailed description which Modbus data and
 * control functions have been implemented in the server engine see section
 * @ref mbusslave.
 *
 * @see mbusslave
 * @see MbusDataTableInterface, MbusRtuSlaveProtocol,
 *      MbusAsciiSlaveProtocol, MbusTcpSlaveProtocol
 */
class
#if defined (_WINDLL) || defined(__DLL__)
   __declspec(dllexport)
#endif
MbusSlaveServer
{

  protected:

   enum
   {
      PDU_SIZE = 253,
      MAX_DATA_SIZE = PDU_SIZE - 1,
      MAX_WORDS = 125,
      MAX_BITS  = 2000
   };

   volatile unsigned long totalCounter;
   volatile unsigned long successCounter;
   long timeOut;   // Time-out in ms
   MbusDataTableInterface *dataTablePtrArr[256];
   MasterInfo masterInfo;


  private:

   short regArr[MAX_WORDS];
   char bitArr[MAX_BITS];
   long enronArr[MAX_WORDS/2];
   int supressExceptionReplies;

  protected:

    MbusSlaveServer();


  public:

   virtual ~MbusSlaveServer();


   /**************************************************************************
    * Server Functions
    **************************************************************************/

   int addDataTable(int slaveAddr, MbusDataTableInterface *dataTablePtr);

   /**
    * Modbus slave server loop
    *
    * This server loop must be called continuously. It must not be
    * blocked. The server has to be started before calling the serverLoop()
    * method.
    *
    * In most cases the server loop is executed in an infinite loop in its own
    * thread:
    * @code
    * while (notTerminated) {
    *   mbusProtocol.serverLoop();
    * }
    * @endcode
    *
    * @return FTALK_SUCCESS on success or error code. See @ref buserror for a
    * list of error codes.
    */
   virtual int serverLoop() = 0;

   /**
    * Shuts down the Modbus server.
    *
    * This function also closes any associated communication resources
    * like serial ports or sockets.
    */
   virtual void shutdownServer() = 0;

   /**
    * Returns whether server has been started up.
    *
    * @retval true = started
    * @retval false = shutdown
   */
   virtual int isStarted() = 0;

   /**
    * Checks if a Modbus master is polling periodically
    *
    * @retval true = A master is polling at a frequency higher than the
    * master transmit time-out value
    * @retval false = No master is polling within the time-out period
    * @note The master transmit time-out value must be set > 0 in order for this
    * function to work.
    */
   virtual int getConnectionStatus() = 0;


   /**************************************************************************
    * Protocol Configuration
    **************************************************************************/

   /**
    * @name Protocol Configuration
    * @ingroup mbusslave
    */
   //@{

  public:

    int setTimeout(long timeOut);


   /**
    * Returns the currently set master time-out supervision value.
    *
    * @return Timeout value in ms
    */
   long getTimeout() { return (timeOut); }


   void disableExceptionReplies();

   void enableExceptionReplies();


   //@}


   /**************************************************************************
    * Transmission Statistic Functions
    **************************************************************************/

   /**
    * @name Transmission Statistic Functions
    * @ingroup mbusslave
    */
   //@{

  public:

   /**
    * Returns how often a message transfer has been executed.
    *
    * @return Counter value
    */
   unsigned long getTotalCounter() { return (totalCounter); }

   /**
    * Resets total message transfer counter.
    */
   void resetTotalCounter() { totalCounter = 0UL; }

   /**
    * Returns how often a message transfer was successful.
    *
    * @return Counter value
    */
   unsigned long getSuccessCounter() { return (successCounter); }

   /**
    * Resets successful message transfer counter.
    */
   void resetSuccessCounter() { successCounter = 0UL; }

   //@}


   /**************************************************************************
    * Utility routines
    **************************************************************************/

  public:

   static const TCHAR *getPackageVersion();


   /**************************************************************************
    * Internal subroutines
    **************************************************************************/

  protected:

   int hasNoDataTable();

   int processMessage(char msgBufArr[], int msgBufCnt);


 private:

   int replyIllegalDataAddress(char msgBufArr[]);

   int replyIllegalValue(char msgBufArr[]);

   int replySlaveFailure(char msgBufArr[]);

   int buildReadDeviceIdResponse(int startObjId, int firstObjId, int lastObjId,
                                 char *msgBufArr);


  private:

   // Disable default operator and copy constructor
   MbusSlaveServer &operator= (MbusSlaveServer &);
   MbusSlaveServer (const MbusSlaveServer &);

};


#endif // ifdef ..._H_INCLUDED
