/**
 * @file MbusTcpSlaveProtocol.hpp
 *
 * @if NOTICE
 *
 * Copyright (c) proconX Pty Ltd. All rights reserved.
 *
 * THIS IS PROPRIETARY SOFTWARE AND YOU NEED A LICENSE TO USE OR REDISTRIBUTE.
 *
 * THIS SOFTWARE IS PROVIDED BY PROCONX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL PROCONX OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * @endif
 */


#ifndef _MBUSTCPSLAVEPROTOCOL_H_INCLUDED
#define _MBUSTCPSLAVEPROTOCOL_H_INCLUDED

#ifndef __cplusplus
#  error Must use C++ to compile this module!
#endif


// Package header
#include "hmtcpip.h"
#include "hmtimer.hpp"
#include "MbusSlaveServer.hpp"


/*****************************************************************************
 * Definitions
 *****************************************************************************/

/**
 * Maximum concurrent TCP/IP connections handled by server engine.
 * This can be increased by re-defining this preprocessor macro
 * (from the comannd line compiler using <code>-DMAX_CONNECTIONS=32</code>).
 *
 * @ingroup mbusslavetcp
 */
#define MAX_CONNECTIONS 16

#if MAX_CONNECTIONS >= (FD_SETSIZE - 1) // 1 for the listener socket
#  error MAX_CONNECTIONS too large!
#endif


/*****************************************************************************
 * MbusTcpSlaveProtocol class declaration
 *****************************************************************************/

/**
 * This class realises the server side of the MODBUS/TCP slave protocol.
 *
 * It provides functions to start-up and to execute the server engine.
 * This server engine can handle multiple master connections and is
 * implemented as a single threaded TCP server.
 * Upon receipt of a valid master query the server engine calls
 * Data Provider methods to exchange data with the user application.
 * For a more detailed description which Modbus data and control functions
 * have been implemented in the server engine seesection @ref mbusslave.
 *
 * @ingroup mbusslavetcp
 * @see mbusslave, mbustcpserial
 * @see MbusDataTableInterface, MbusRtuSlaveProtocol, MbusAsciiSlaveProtocol
 */
class
#if defined (_WINDLL) || defined(__DLL__)
   __declspec(dllexport)
#endif
MbusTcpSlaveProtocol: public MbusSlaveServer
{

  protected:

   enum
   {
      // MBAP prefix
      MBAP_HDR_LEN = 6,
      // Header is MBAP prefix + 1 byte function + 1 byte address
      HDR_LEN = MBAP_HDR_LEN + 2,
      // Frame is same as header for this protocol
      FRAME_LEN = HDR_LEN,
      // TCP max. message size is data size + frame length:
      MAX_MSG_SIZE = MAX_DATA_SIZE + FRAME_LEN
   };

   char bufferArr[MAX_MSG_SIZE];


  public:

   MbusTcpSlaveProtocol();

   ~MbusTcpSlaveProtocol();

   /**
    * @name TCP/IP Server Management Functions
    */
   //@{

   int startupServer();

   int startupServer(const char * const hostName);

   void shutdownServer();

   int serverLoop();

   int isStarted();

   int getConnectionStatus();

   int setPort(unsigned short portNo);

   int setConnectionTimeOut(long masterTimeOut);

   void installIpAddrValidationCallBack(int (*f) (const char * masterIpAddrSz));

   void installMasterPollNotifyCallBack(void (*f) (const char * masterIpAddrSz));

   void installMasterDisconnectCallBack(void (*f) (const char * masterIpAddrSz));


   /**
    * Returns the TCP port number used by the protocol.
    *
    * @return Port number used by the protocol
    */
   unsigned short getPort() { return (portNo); }


   /**
    * Returns the connection time-out setting.
    *
    * @return Timeout value in ms
    */
   long getConnectionTimeOut() { return (masterTimeOut); }


   //@}


  protected:

   SOCKET listenSocket;
   SOCKET connectionSocketArr[MAX_CONNECTIONS];
   struct sockaddr_in peerAddrArr[MAX_CONNECTIONS];
   HmTimer toTimer[MAX_CONNECTIONS];
   unsigned short portNo; // TCP/IP port number
   int masterTimeOut;
   int masterIsConnected;

   int replyMessage(char msgBufArr[], int msgBufLen);
   int (*validateMasterIpAddrFunc) (const char * masterIpAddrSz);
   void (*masterPollNotifyFunc) (const char * masterIpAddrSz);
   void (*masterDisconnectNotifyFunc) (const char * masterIpAddrSz);


  private:

   // Disable default operator and copy constructor
   MbusTcpSlaveProtocol &operator= (MbusTcpSlaveProtocol &);
   MbusTcpSlaveProtocol (const MbusTcpSlaveProtocol &);

};


#endif // ifdef ..._H_INCLUDED
